@@ -1,4 +1,4 @@
-// (C) 2008 D. V. Wiebe
+// Copyright (C) 2008, 2010, 2011 D. V. Wiebe
 //
 ///////////////////////////////////////////////////////////////////////////
 //
@@ -18,9 +18,7 @@
 // along with GetData; if not, write to the Free Software Foundation, Inc.,
 // 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
-#include "getdata/dirfile.h"
-
-#include <cstring>
+#include "internal.h"
 
 using namespace GetData;
 
@@ -29,16 +27,16 @@ ConstEntry::ConstEntry(const char* field_code, DataType data_type,
 {
   E.field = strdup(field_code);
   E.field_type = GD_CONST_ENTRY;
-  E.const_type = (gd_type_t)data_type;
+  E.u.scalar.const_type = (gd_type_t)data_type;
   E.fragment_index = fragment_index;
 }
 
 int ConstEntry::SetType(DataType type)
 {
-  E.const_type = (gd_type_t)type;
+  E.u.scalar.const_type = (gd_type_t)type;
 
   if (D != NULL)
-    return dirfile_alter_entry(D->D, E.field, &E, 0);
+    return gd_alter_entry(D->D, E.field, &E, 0);
   
   return 0;
 }
