@@ -1,4 +1,4 @@
-// (C) 2008 D. V. Wiebe
+// Copyright (C) 2011 D. V. Wiebe
 //
 ///////////////////////////////////////////////////////////////////////////
 //
@@ -18,8 +18,4 @@
 // along with GetData; if not, write to the Free Software Foundation, Inc.,
 // 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
-#include "getdata/dirfile.h"
-
-#include <cstring>
-
-using namespace GetData;
+#include "internal.h"
