@@ -1,4 +1,4 @@
-// (C) 2008 D. V. Wiebe
+// Copyright (C) 2008, 2010, 2011 D. V. Wiebe
 //
 ///////////////////////////////////////////////////////////////////////////
 //
@@ -18,9 +18,7 @@
 // along with GetData; if not, write to the Free Software Foundation, Inc.,
 // 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
-#include "getdata/dirfile.h"
-
-#include <cstring>
+#include "internal.h"
 
 using namespace GetData;
 
