@@ -1,4 +1,4 @@
-/* (C) 2009 D. V. Wiebe
+/* Copyright (C) 2009-2013 D. V. Wiebe
  *
  ***************************************************************************
  *
@@ -30,8 +30,8 @@ static const char *gdpy_exception_list[GD_N_ERROR_CODES] = {
   "BadCode",
   "BadType",
   "RawIO",
-  "OpenInclude",
-  "InternalError",
+  "OpenFragment",
+  "Internal",
   "Alloc",
   "Range",
   "OpenLinfile",
@@ -49,21 +49,26 @@ static const char *gdpy_exception_list[GD_N_ERROR_CODES] = {
   "BadReference",
   "Protected",
   "Deletion",
-  "BadEndianess",
+  "Argument",
   "Callback",
-  "BadProtection",
+  "Exists",
   "UncleanDatabase",
   "Domain",
-  "BadRepr"
+  "BadRepr",
+  NULL,
+  "Flush",
+  "Bounds",
+  "LineTooLong"
 };
 PyObject *gdpy_exceptions[GD_N_ERROR_CODES];
 
-int gdpy_convert_from_pyobj(PyObject* value, union gdpy_quadruple_value *data,
+int gdpy_convert_from_pyobj(PyObject *value, union gdpy_quadruple_value *data,
     gd_type_t type)
 {
-  dtrace("%p, %p, %02x", value, data, type);
   int data_type;
 
+  dtrace("%p, %p, %02x", value, data, type);
+
   /* check value type, and figure out autotype, if needed */
   if (PyInt_Check(value)) {
     data->s = PyInt_AsLong(value);
@@ -145,14 +150,14 @@ int gdpy_convert_from_pyobj(PyObject* value, union gdpy_quadruple_value *data,
       return -1;
     }
   } else if (PyComplex_Check(value)) {
-    data->c = gdpy_as_complex(value);
+    gdpy_as_complex(gd_csp_(data->c), value);
     data_type = GDPY_COMPLEX_AS_COMPLEX;
 
     if (PyErr_Occurred()) {
       dreturn("%i", -1);
       return -1;
     }
-  } else { /* an non numeric type */
+  } else { /* a non-numeric type */
     PyErr_SetString(PyExc_TypeError, "a numeric type was expected");
     dreturn("%i", -1);
     return -1;
@@ -162,16 +167,17 @@ int gdpy_convert_from_pyobj(PyObject* value, union gdpy_quadruple_value *data,
   return data_type;
 }
 
-gd_type_t gdpy_convert_from_pylist(PyObject* value, void *data, gd_type_t type,
+gd_type_t gdpy_convert_from_pylist(PyObject *value, void *data, gd_type_t type,
     size_t ns)
 {
+  size_t i;
+  int data_type;
+  union gdpy_quadruple_value tmp;
+
   dtrace("%p, %p, %02x, %zi", value, data, type, ns);
 
-  size_t i;
-
   /* use the first element to determine the data type */
-  int data_type = gdpy_convert_from_pyobj(PyList_GetItem(value, 0),
-      (union gdpy_quadruple_value*)data, type);
+  data_type = gdpy_convert_from_pyobj(PyList_GetItem(value, 0), &tmp, type);
 
   if (data_type == -1) {
     dreturn("%02x", GD_UNKNOWN);
@@ -180,35 +186,41 @@ gd_type_t gdpy_convert_from_pylist(PyObject* value, void *data, gd_type_t type,
 
   switch(data_type) {
     case GDPY_INT_AS_LONG:
-      type = GD_INT64;
+      type = GD_INT32;
+      *(int32_t*)data = tmp.s;
       for (i = 1; i < ns; ++i)
-        ((int64_t*)data)[i] = PyInt_AsLong(PyList_GetItem(value, i));
+        ((int32_t*)data)[i] = PyInt_AsLong(PyList_GetItem(value, i));
       break;
     case GDPY_LONG_AS_ULL: 
       type = GD_UINT64;
+      *(uint64_t*)data = tmp.u;
       for (i = 1; i < ns; ++i)
         ((uint64_t*)data)[i] = PyLong_AsUnsignedLongLong(PyList_GetItem(value,
             i));
       break;
     case GDPY_LONG_AS_SLL: 
       type = GD_INT64;
+      *(int64_t*)data = tmp.s;
       for (i = 1; i < ns; ++i)
         ((int64_t*)data)[i] = PyLong_AsLongLong(PyList_GetItem(value, i));
       break;
     case GDPY_LONG_AS_DOUBLE: 
       type = GD_FLOAT64;
+      *(double*)data = tmp.f;
       for (i = 1; i < ns; ++i)
         ((double*)data)[i] = PyLong_AsDouble(PyList_GetItem(value, i));
       break;
     case GDPY_FLOAT_AS_DOUBLE:
       type = GD_FLOAT64;
+      *(double*)data = tmp.f;
       for (i = 1; i < ns; ++i)
         ((double*)data)[i] = PyFloat_AsDouble(PyList_GetItem(value, i));
       break;
     case GDPY_COMPLEX_AS_COMPLEX:
       type = GD_COMPLEX128;
+      gd_cs2ca_(data, 0, tmp.c, double);
       for (i = 1; i < ns; ++i)
-        ((double complex*)data)[i] = gdpy_as_complex(PyList_GetItem(value, i));
+        gdpy_as_complex(((double*)data) + 2 * i, PyList_GetItem(value, i));
       break;
   }
 
@@ -234,50 +246,50 @@ gd_type_t gdpy_type_from_npytype(int npytype)
       break;
 #if NPY_SIZEOF_SHORT <= 8
     case NPY_SHORT:
-      type = NPY_SIZEOF_SHORT | GD_SIGNED;
+      type = (gd_type_t)(NPY_SIZEOF_SHORT | GD_SIGNED);
       break;
     case NPY_USHORT:
-      type = NPY_SIZEOF_SHORT;
+      type = (gd_type_t)NPY_SIZEOF_SHORT;
       break;
 #endif
 #if NPY_SIZEOF_INT <= 8
     case NPY_INT:
-      type = NPY_SIZEOF_INT | GD_SIGNED;
+      type = (gd_type_t)(NPY_SIZEOF_INT | GD_SIGNED);
       break;
     case NPY_UINT:
-      type = NPY_SIZEOF_INT;
+      type = (gd_type_t)NPY_SIZEOF_INT;
       break;
 #endif
 #if NPY_SIZEOF_LONG <= 8
     case NPY_LONG:
-      type = NPY_SIZEOF_LONG | GD_SIGNED;
+      type = (gd_type_t)(NPY_SIZEOF_LONG | GD_SIGNED);
       break;
     case NPY_ULONG:
-      type = NPY_SIZEOF_LONG;
+      type = (gd_type_t)NPY_SIZEOF_LONG;
       break;
 #endif
 #if NPY_SIZEOF_LONGLONG <= 8
     case NPY_LONGLONG:
-      type = NPY_SIZEOF_LONGLONG | GD_SIGNED;
+      type = (gd_type_t)(NPY_SIZEOF_LONGLONG | GD_SIGNED);
       break;
     case NPY_ULONGLONG:
-      type = NPY_SIZEOF_LONGLONG;
+      type = (gd_type_t)NPY_SIZEOF_LONGLONG;
       break;
 #endif
 #if NPY_SIZEOF_FLOAT <= 8
     case NPY_FLOAT:
-      type = NPY_SIZEOF_FLOAT | GD_IEEE754;
+      type = (gd_type_t)NPY_SIZEOF_FLOAT | GD_IEEE754;
       break;
     case NPY_CFLOAT:
-      type = (2 * NPY_SIZEOF_FLOAT) | GD_COMPLEX;
+      type = (gd_type_t)((2 * NPY_SIZEOF_FLOAT) | GD_COMPLEX);
       break;
 #endif
 #if NPY_SIZEOF_DOUBLE <= 8
     case NPY_DOUBLE:
-      type = NPY_SIZEOF_DOUBLE | GD_IEEE754;
+      type = (gd_type_t)(NPY_SIZEOF_DOUBLE | GD_IEEE754);
       break;
     case NPY_CDOUBLE:
-      type = (2 * NPY_SIZEOF_DOUBLE) | GD_COMPLEX;
+      type = (gd_type_t)((2 * NPY_SIZEOF_DOUBLE) | GD_COMPLEX);
       break;
 #endif
     default:
@@ -343,16 +355,15 @@ int gdpy_npytype_from_type(gd_type_t type)
 }
 #endif
 
-PyObject* gdpy_convert_to_pylist(const void* data, gd_type_t type, size_t ns)
+PyObject *gdpy_convert_to_pylist(const void *data, gd_type_t type, size_t ns)
 {
   size_t i;
+  PyObject *pyobj;
 
   dtrace("%p, %02x, %zi", data, type, ns);
 
-  PyObject* pylist;
-
   if (type != GD_NULL)
-    pylist = PyList_New(0);
+    pyobj = PyList_New(0);
 
   switch(type) {
     case GD_NULL:
@@ -361,82 +372,80 @@ PyObject* gdpy_convert_to_pylist(const void* data, gd_type_t type, size_t ns)
       return Py_None;
     case GD_UINT8:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist, PyInt_FromLong((long)((uint8_t*)data)[i])))
+        if (PyList_Append(pyobj, PyInt_FromLong((long)((uint8_t*)data)[i])))
           return NULL;
       break;
     case GD_INT8:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist, PyInt_FromLong((long)((int8_t*)data)[i])))
+        if (PyList_Append(pyobj, PyInt_FromLong((long)((int8_t*)data)[i])))
           return NULL;
       break;
     case GD_UINT16:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist, PyInt_FromLong((long)((uint16_t*)data)[i])))
+        if (PyList_Append(pyobj, PyInt_FromLong((long)((uint16_t*)data)[i])))
           return NULL;
       break;
     case GD_INT16:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist, PyInt_FromLong((long)((int16_t*)data)[i])))
+        if (PyList_Append(pyobj, PyInt_FromLong((long)((int16_t*)data)[i])))
           return NULL;
       break;
     case GD_UINT32:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist,
+        if (PyList_Append(pyobj,
               PyLong_FromUnsignedLong((unsigned long)((uint32_t*)data)[i])))
           return NULL;
       break;
     case GD_INT32:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist, PyInt_FromLong((long)((int32_t*)data)[i])))
+        if (PyList_Append(pyobj, PyInt_FromLong((long)((int32_t*)data)[i])))
           return NULL;
       break;
     case GD_UINT64:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist, PyLong_FromUnsignedLongLong(
+        if (PyList_Append(pyobj, PyLong_FromUnsignedLongLong(
                 (unsigned long long)((uint64_t*)data)[i])))
           return NULL;
       break;
     case GD_INT64:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist,
+        if (PyList_Append(pyobj,
               PyLong_FromLongLong((long long)((int64_t*)data)[i])))
           return NULL;
       break;
     case GD_FLOAT32:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist,
-              PyFloat_FromDouble((double)((float*)data)[i])))
+        if (PyList_Append(pyobj, PyFloat_FromDouble((double)((float*)data)[i])))
           return NULL;
       break;
     case GD_FLOAT64:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist, PyFloat_FromDouble(((double*)data)[i])))
+        if (PyList_Append(pyobj, PyFloat_FromDouble(((double*)data)[i])))
           return NULL;
       break;
     case GD_COMPLEX64:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist,
-              gdpy_from_complex((double complex)((float complex*)data)[i])))
+        if (PyList_Append(pyobj, gdpy_from_complexp(((float*)data) + 2 * i)))
           return NULL;
       break;
     case GD_COMPLEX128:
       for (i = 0; i < ns; ++i)
-        if (PyList_Append(pylist,
-              gdpy_from_complex(((double complex*)data)[i])))
+        if (PyList_Append(pyobj, gdpy_from_complexp(((double*)data) + 2 * i)))
           return NULL;
       break;
     case GD_UNKNOWN: /* prevent compiler warning */
       break;
   }
 
-  dreturn("%p", pylist);
-  return pylist;
+  dreturn("%p", pyobj);
+  return pyobj;
 }
 
-PyObject* gdpy_convert_to_pyobj(const void* data, gd_type_t type)
+PyObject *gdpy_convert_to_pyobj(const void *data, gd_type_t type)
 {
+  PyObject *pyobj = NULL;
+
   dtrace("%p, %02x", data, type);
-  PyObject* pyobj = NULL;
 
   switch(type) {
     case GD_NULL:
@@ -474,10 +483,10 @@ PyObject* gdpy_convert_to_pyobj(const void* data, gd_type_t type)
       pyobj = PyFloat_FromDouble(*(double*)data);
       break;
     case GD_COMPLEX64:
-      pyobj = gdpy_from_complex((double complex)*(float complex*)data);
+      pyobj = gdpy_from_complexp((float*)data);
       break;
     case GD_COMPLEX128:
-      pyobj = gdpy_from_complex(*(double complex*)data);
+      pyobj = gdpy_from_complexp((double*)data);
       break;
     case GD_UNKNOWN: /* prevent compiler warning */
       break;
@@ -495,21 +504,27 @@ static PyMethodDef GetDataMethods[] = {
 PyMODINIT_FUNC initpygetdata(void)
 {
   int i;
-  PyObject* mod;
+  PyObject *mod;
+
+  dtracevoid();
 
   if (PyType_Ready(&gdpy_dirfile) < 0)
     return;
+  dprintf("gdpy_dirfile ready");
 
   if (PyType_Ready(&gdpy_entry) < 0)
     return;
+  dprintf("gdpy_entry ready");
 
   if (PyType_Ready(&gdpy_fragment) < 0)
     return;
+  dprintf("gdpy_fragment ready");
 
 #ifdef USE_NUMPY
   /* The following macro will cause this function to return if importing numpy
    * fails */
   import_array()
+  dprintf("imported NumPy");
 #endif
 
   mod = Py_InitModule3("pygetdata", GetDataMethods,
@@ -537,13 +552,13 @@ PyMODINIT_FUNC initpygetdata(void)
       "not\npresent, vector data will be returned as Python lists.  Vector "
       "data passed\nto pygetdata may either be a Python list or a NumPy array."
       "\n\n"
-      "The input data type argument to bindings for functions such as "
-      "putdata(3),\nwhich is required in the C API, are typically optional, as "
-      "pygetdata can\ndetermine the input data type by itself, and convert it "
-      "to an appropriate\ntype for the C API.  If the data type is supplied, "
-      "pygetdata will coerce the\ninput data to the specified C type as best "
-      "it can.  For getdata(3) and\nsimilar, the C API types are converted to "
-      "Python types as follows:\n\n"
+      "The input data type argument to bindings for functions such as\n"
+      "gd_putdata(3), which is required in the C API, are typically optional,\n"
+      "as pygetdata can determine the input data type by itself, and convert "
+      "it to\nan appropriate type for the C API.  If the data type is supplied,"
+      " pygetdata\nwill coerce the input data to the specified C type as best "
+      "it can.  For\ngd_getdata(3) and similar, the C API types are converted "
+      "to Python types as\nfollows:\n\n"
       "  o int     -- UINT8, INT8, UINT16, INT16, INT32\n"
       "  o long    -- UINT32, UINT64, INT64\n"
       "  o float   -- FLOAT32, FLOAT64\n"
@@ -562,28 +577,35 @@ PyMODINIT_FUNC initpygetdata(void)
 
   if (mod == NULL)
     return;
+  dprintf("module init");
 
   Py_INCREF(&gdpy_dirfile);
   PyModule_AddObject(mod, "dirfile", (PyObject *)&gdpy_dirfile);
+  dprintf("gdpy_dirfile added");
 
   Py_INCREF(&gdpy_entry);
   PyModule_AddObject(mod, "entry", (PyObject *)&gdpy_entry);
+  dprintf("gdpy_entry added");
 
   Py_INCREF(&gdpy_fragment);
   PyModule_AddObject(mod, "fragment", (PyObject *)&gdpy_fragment);
+  dprintf("gdpy_fragment added");
 
   /* version */
   PyModule_AddObject(mod, "__version__", Py_BuildValue("(iiis)", GETDATA_MAJOR,
         GETDATA_MINOR, GETDATA_REVISION, GETDATA_VERSION_SUFFIX));
+  dprintf(".__version__ added");
 
   /* author */
   PyModule_AddStringConstant(mod, "__author__",
       "D. V. Wiebe <getdata@ketiltrout.net>");
+  dprintf(".__author__ added");
 
   /* add constants */
   for (i = 0; gdpy_constant_list[i].name != NULL; ++i)
     PyModule_AddIntConstant(mod, gdpy_constant_list[i].name,
         gdpy_constant_list[i].value);
+  dprintf("constants added");
 
   PyModule_AddIntConstant(mod, "__numpy_supported__",
 #ifdef USE_NUMPY
@@ -592,6 +614,7 @@ PyMODINIT_FUNC initpygetdata(void)
       0
 #endif
       );
+  dprintf(".__numpy_supported__ added");
 
   /* add exceptions */
   GdPy_DirfileError = PyErr_NewException("pygetdata.DirfileError",
@@ -600,11 +623,16 @@ PyMODINIT_FUNC initpygetdata(void)
   PyModule_AddObject(mod, "DirfileError", GdPy_DirfileError);
 
   for (i = 1; i < GD_N_ERROR_CODES; ++i) {
-    char name[40];
-    sprintf(name, "pygetdata.%sError", gdpy_exception_list[i]);
-    gdpy_exceptions[i] = PyErr_NewException(name, GdPy_DirfileError, NULL);
-    Py_INCREF(gdpy_exceptions[i]);
-    sprintf(name, "%sError", gdpy_exception_list[i]);
-    PyModule_AddObject(mod, name, gdpy_exceptions[i]);
+    if (gdpy_exception_list[i]) {
+      char name[40];
+      sprintf(name, "pygetdata.%sError", gdpy_exception_list[i]);
+      gdpy_exceptions[i] = PyErr_NewException(name, GdPy_DirfileError, NULL);
+      Py_INCREF(gdpy_exceptions[i]);
+      PyModule_AddObject(mod, name + 10, gdpy_exceptions[i]);
+    } else
+      gdpy_exceptions[i] = GdPy_DirfileError;
   }
+  dprintf("Exceptions added");
+
+  dreturnvoid();
 }
