@@ -1,4 +1,4 @@
-/* (C) 2008 D. V. Wiebe
+/* Copyright (C) 2008, 2010, 2013 D. V. Wiebe
  *
  ***************************************************************************
  *
@@ -20,35 +20,36 @@
  */
 #include "internal.h"
 
-#include <string.h>
-#include "getdata.h"
-#include "internal.h"
+extern char gd_debug_col[GD_COL_SIZE + 1];
+extern int gd_col_count;
 
-#define COL_SIZE 100
-static char debug_col[COL_SIZE + 1] = "";
-static int col_count = 0;
-
-const char* _gd_colnil(void) {
-  return debug_col;
+const char* gd_colnil(void) {
+  return gd_debug_col;
 }
 
-const char* _gd_coladd(void)
+const char* gd_coladd(void)
 {
-  if (col_count < COL_SIZE) {
-    debug_col[col_count++] = ':';
-    debug_col[col_count] = '\0';
+  if (gd_col_count < GD_COL_SIZE) {
+    gd_debug_col[gd_col_count++] = ':';
+    gd_debug_col[gd_col_count] = '\0';
   }
 
-  return _gd_colnil();
+  return gd_colnil();
 }
 
-const char* _gd_colsub(void)
+const char* gd_colsub(void)
 {
-  static char buffer[COL_SIZE + 1];
-  strcpy(buffer, _gd_colnil());
+  static char buffer[GD_COL_SIZE + 1];
+  strcpy(buffer, gd_colnil());
 
-  if (col_count > 0)
-    debug_col[--col_count] = '\0';
+  if (gd_col_count > 0)
+    gd_debug_col[--gd_col_count] = '\0';
 
   return buffer;
 }
+
+void gd_colclear(void)
+{
+  gd_col_count = 0;
+  gd_debug_col[0] = '\0';
+}
