@@ -1,5 +1,5 @@
-/* (C) 2002-2005 C. Barth Netterfield
- * (C) 2005-2009 D. V. Wiebe
+/* Copyright (C) 2002-2005 C. Barth Netterfield
+ * Copyright (C) 2005-2010 D. V. Wiebe
  *
  ***************************************************************************
  *
@@ -21,24 +21,50 @@
  */
 #include "internal.h"
 
-#ifdef STDC_HEADERS
-#include <inttypes.h>
-#include <errno.h>
-#include <fcntl.h>
-#include <math.h>
-#include <stdlib.h>
-#include <string.h>
+#ifdef GD_NO_C99_API
+
+#define TO_COMPLEX(ot,it) \
+  do { \
+    for (i = 0; i < n; i++) { \
+      ((ot *)data_out)[2 * i] = (ot)((it *)data_in)[i]; \
+      ((ot *)data_out)[2 * i + 1] = 0; \
+    } \
+  } while (0)
+#define FROM_COMPLEX(ot,it) \
+  do { \
+    for (i = 0; i < n; i++) { \
+      ((ot *)data_out)[i] = (ot)((it *)data_in)[2 * i]; \
+    } \
+  } while (0)
+
+#else
+
+#define TO_COMPLEX(ot,it) \
+  do { \
+    for (i = 0; i < n; i++) { \
+      ((complex ot *)data_out)[i] = (complex ot)((it *)data_in)[i]; \
+    } \
+  } while (0)
+#define FROM_COMPLEX(ot,it) \
+  do { \
+    for (i = 0; i < n; i++) { \
+      ((ot *)data_out)[i] = (ot)((complex it *)data_in)[i]; \
+    } \
+  } while (0)
+
 #endif
 
+
 /* _GD_ConvertType: copy data to output buffer while converting type.
 */
-void _GD_ConvertType(DIRFILE* D, const void *data_in, gd_type_t in_type,
-    void *data_out, gd_type_t out_type, size_t n)
+void _GD_ConvertType(DIRFILE *restrict D, const void *restrict data_in,
+    gd_type_t in_type, void *restrict data_out, gd_type_t out_type, size_t n)
+gd_nothrow
 {
   size_t i;
 
-  dtrace("%p, %p, 0x%x, %p, 0x%x, %zi", D, data_in, in_type, data_out, out_type,
-      n);
+  dtrace("%p, %p, 0x%x, %p, 0x%x, %" PRNsize_t, D, data_in, in_type, data_out,
+      out_type, n);
   dreturnvoid();
 
   if (out_type == GD_NULL) /* null return type: don't return data */
@@ -52,49 +78,45 @@ void _GD_ConvertType(DIRFILE* D, const void *data_in, gd_type_t in_type,
           return;
         case GD_UINT8:
           for (i = 0; i < n; i++)
-            ((uint8_t*)data_out)[i] = (uint8_t)((int8_t*)data_in)[i];
+            ((uint8_t *)data_out)[i] = (uint8_t)((int8_t *)data_in)[i];
           return;
         case GD_INT16:
           for (i = 0; i < n; i++)
-            ((int16_t*)data_out)[i] = (int16_t)((int8_t*)data_in)[i];
+            ((int16_t *)data_out)[i] = (int16_t)((int8_t *)data_in)[i];
           return;
         case GD_UINT16:
           for (i = 0; i < n; i++)
-            ((uint16_t*)data_out)[i] = (uint16_t)((int8_t*)data_in)[i];
+            ((uint16_t *)data_out)[i] = (uint16_t)((int8_t *)data_in)[i];
           return;
         case GD_INT32:
           for (i = 0; i < n; i++)
-            ((int32_t*)data_out)[i] = (int32_t)((int8_t*)data_in)[i];
+            ((int32_t *)data_out)[i] = (int32_t)((int8_t *)data_in)[i];
           return;
         case GD_UINT32:
           for (i = 0; i < n; i++)
-            ((uint32_t*)data_out)[i] = (uint32_t)((int8_t*)data_in)[i];
+            ((uint32_t *)data_out)[i] = (uint32_t)((int8_t *)data_in)[i];
           return;
         case GD_INT64:
           for (i = 0; i < n; i++)
-            ((int64_t*)data_out)[i] = (int64_t)((int8_t*)data_in)[i];
+            ((int64_t *)data_out)[i] = (int64_t)((int8_t *)data_in)[i];
           return;
         case GD_UINT64:
           for (i = 0; i < n; i++)
-            ((uint64_t*)data_out)[i] = (uint64_t)((int8_t*)data_in)[i];
+            ((uint64_t *)data_out)[i] = (uint64_t)((int8_t *)data_in)[i];
           return;
         case GD_FLOAT32:
           for (i = 0; i < n; i++)
-            ((float*)data_out)[i] = (float)((int8_t*)data_in)[i];
+            ((float *)data_out)[i] = (float)((int8_t *)data_in)[i];
           return;
         case GD_FLOAT64:
           for (i = 0; i < n; i++)
-            ((double*)data_out)[i] = (double)((int8_t*)data_in)[i];
+            ((double *)data_out)[i] = (double)((int8_t *)data_in)[i];
           return;
         case GD_COMPLEX64:
-          for (i = 0; i < n; i++)
-            ((float complex*)data_out)[i] =
-              (float complex)((int8_t*)data_in)[i];
+          TO_COMPLEX(float,int8_t);
           return;
         case GD_COMPLEX128:
-          for (i = 0; i < n; i++)
-            ((double complex*)data_out)[i] =
-              (double complex)((int8_t*)data_in)[i];
+          TO_COMPLEX(double,int8_t);
           return;
         default:
           _GD_SetError(D, GD_E_BAD_TYPE, out_type, NULL, 0, NULL);
@@ -105,52 +127,48 @@ void _GD_ConvertType(DIRFILE* D, const void *data_in, gd_type_t in_type,
       switch (out_type) {
         case GD_INT8:
           for (i = 0; i < n; i++)
-            ((int8_t*)data_out)[i] = (int8_t)((uint8_t*)data_in)[i];
+            ((int8_t *)data_out)[i] = (int8_t)((uint8_t *)data_in)[i];
           return;
         case GD_UINT8:
           memcpy(data_out, data_in, n * sizeof(uint8_t));
           return;
         case GD_INT16:
           for (i = 0; i < n; i++)
-            ((int16_t*)data_out)[i] = (int16_t)((uint8_t*)data_in)[i];
+            ((int16_t *)data_out)[i] = (int16_t)((uint8_t *)data_in)[i];
           return;
         case GD_UINT16:
           for (i = 0; i < n; i++)
-            ((uint16_t*)data_out)[i] = (uint16_t)((uint8_t*)data_in)[i];
+            ((uint16_t *)data_out)[i] = (uint16_t)((uint8_t *)data_in)[i];
           return;
         case GD_INT32:
           for (i = 0; i < n; i++)
-            ((int32_t*)data_out)[i] = (int32_t)((uint8_t*)data_in)[i];
+            ((int32_t *)data_out)[i] = (int32_t)((uint8_t *)data_in)[i];
           return;
         case GD_UINT32:
           for (i = 0; i < n; i++)
-            ((uint32_t*)data_out)[i] = (uint32_t)((uint8_t*)data_in)[i];
+            ((uint32_t *)data_out)[i] = (uint32_t)((uint8_t *)data_in)[i];
           return;
         case GD_INT64:
           for (i = 0; i < n; i++)
-            ((int64_t*)data_out)[i] = (int64_t)((uint8_t*)data_in)[i];
+            ((int64_t *)data_out)[i] = (int64_t)((uint8_t *)data_in)[i];
           return;
         case GD_UINT64:
           for (i = 0; i < n; i++)
-            ((uint64_t*)data_out)[i] = (uint64_t)((uint8_t*)data_in)[i];
+            ((uint64_t *)data_out)[i] = (uint64_t)((uint8_t *)data_in)[i];
           return;
         case GD_FLOAT32:
           for (i = 0; i < n; i++)
-            ((float*)data_out)[i] = (float)((uint8_t*)data_in)[i];
+            ((float *)data_out)[i] = (float)((uint8_t *)data_in)[i];
           return;
         case GD_FLOAT64:
           for (i = 0; i < n; i++)
-            ((double*)data_out)[i] = (double)((uint8_t*)data_in)[i];
+            ((double *)data_out)[i] = (double)((uint8_t *)data_in)[i];
           return;
         case GD_COMPLEX64:
-          for (i = 0; i < n; i++)
-            ((float complex*)data_out)[i] =
-              (float complex)((uint8_t*)data_in)[i];
+          TO_COMPLEX(float,uint8_t);
           return;
         case GD_COMPLEX128:
-          for (i = 0; i < n; i++)
-            ((double complex*)data_out)[i] =
-              (double complex)((uint8_t*)data_in)[i];
+          TO_COMPLEX(double,uint8_t);
           return;
         default:
           _GD_SetError(D, GD_E_BAD_TYPE, out_type, NULL, 0, NULL);
@@ -161,52 +179,48 @@ void _GD_ConvertType(DIRFILE* D, const void *data_in, gd_type_t in_type,
       switch (out_type) {
         case GD_INT8:
           for (i = 0; i < n; i++)
-            ((int8_t*)data_out)[i] = (int8_t)((int16_t*)data_in)[i];
+            ((int8_t *)data_out)[i] = (int8_t)((int16_t *)data_in)[i];
           return;
         case GD_UINT8:
           for (i = 0; i < n; i++)
-            ((uint8_t*)data_out)[i] = (uint8_t)((int16_t*)data_in)[i];
+            ((uint8_t *)data_out)[i] = (uint8_t)((int16_t *)data_in)[i];
           return;
         case GD_INT16:
           memcpy(data_out, data_in, n * sizeof(int16_t));
           return;
         case GD_UINT16:
           for (i = 0; i < n; i++)
-            ((uint16_t*)data_out)[i] = (uint16_t)((int16_t*)data_in)[i];
+            ((uint16_t *)data_out)[i] = (uint16_t)((int16_t *)data_in)[i];
           return;
         case GD_INT32:
           for (i = 0; i < n; i++)
-            ((int32_t*)data_out)[i] = (int32_t)((int16_t*)data_in)[i];
+            ((int32_t *)data_out)[i] = (int32_t)((int16_t *)data_in)[i];
           return;
         case GD_UINT32:
           for (i = 0; i < n; i++)
-            ((uint32_t*)data_out)[i] = (uint32_t)((uint16_t*)data_in)[i];
+            ((uint32_t *)data_out)[i] = (uint32_t)((uint16_t *)data_in)[i];
           return;
         case GD_INT64:
           for (i = 0; i < n; i++)
-            ((int64_t*)data_out)[i] = (int64_t)((int16_t*)data_in)[i];
+            ((int64_t *)data_out)[i] = (int64_t)((int16_t *)data_in)[i];
           return;
         case GD_UINT64:
           for (i = 0; i < n; i++)
-            ((int64_t*)data_out)[i] = (uint64_t)((int16_t*)data_in)[i];
+            ((int64_t *)data_out)[i] = (uint64_t)((int16_t *)data_in)[i];
           return;
         case GD_FLOAT32:
           for (i = 0; i < n; i++)
-            ((float*)data_out)[i] = (float)((int16_t*)data_in)[i];
+            ((float *)data_out)[i] = (float)((int16_t *)data_in)[i];
           return;
         case GD_FLOAT64:
           for (i = 0; i < n; i++)
-            ((double*)data_out)[i] = (double)((int16_t*)data_in)[i];
+            ((double *)data_out)[i] = (double)((int16_t *)data_in)[i];
           return;
         case GD_COMPLEX64:
-          for (i = 0; i < n; i++)
-            ((float complex*)data_out)[i] =
-              (float complex)((int16_t*)data_in)[i];
+          TO_COMPLEX(float,int16_t);
           return;
         case GD_COMPLEX128:
-          for (i = 0; i < n; i++)
-            ((double complex*)data_out)[i] =
-              (double complex)((int16_t*)data_in)[i];
+          TO_COMPLEX(double,int16_t);
           return;
         default:
           _GD_SetError(D, GD_E_BAD_TYPE, out_type, NULL, 0, NULL);
@@ -217,52 +231,48 @@ void _GD_ConvertType(DIRFILE* D, const void *data_in, gd_type_t in_type,
       switch (out_type) {
         case GD_INT8:
           for (i = 0; i < n; i++)
-            ((int8_t*)data_out)[i] = (int8_t)((uint16_t*)data_in)[i];
+            ((int8_t *)data_out)[i] = (int8_t)((uint16_t *)data_in)[i];
           return;
         case GD_UINT8:
           for (i = 0; i < n; i++)
-            ((uint8_t*)data_out)[i] = (uint8_t)((uint16_t*)data_in)[i];
+            ((uint8_t *)data_out)[i] = (uint8_t)((uint16_t *)data_in)[i];
           return;
         case GD_INT16:
           for (i = 0; i < n; i++)
-            ((int16_t*)data_out)[i] = (int16_t)((uint16_t*)data_in)[i];
+            ((int16_t *)data_out)[i] = (int16_t)((uint16_t *)data_in)[i];
           return;
         case GD_UINT16:
           memcpy(data_out, data_in, n * sizeof(uint16_t));
           return;
         case GD_INT32:
           for (i = 0; i < n; i++)
-            ((int32_t*)data_out)[i] = (int32_t)((uint16_t*)data_in)[i];
+            ((int32_t *)data_out)[i] = (int32_t)((uint16_t *)data_in)[i];
           return;
         case GD_UINT32:
           for (i = 0; i < n; i++)
-            ((uint32_t*)data_out)[i] = (uint32_t)((uint16_t*)data_in)[i];
+            ((uint32_t *)data_out)[i] = (uint32_t)((uint16_t *)data_in)[i];
           return;
         case GD_INT64:
           for (i = 0; i < n; i++)
-            ((int64_t*)data_out)[i] = (int64_t)((uint16_t*)data_in)[i];
+            ((int64_t *)data_out)[i] = (int64_t)((uint16_t *)data_in)[i];
           return;
         case GD_UINT64:
           for (i = 0; i < n; i++)
-            ((uint64_t*)data_out)[i] = (uint64_t)((uint16_t*)data_in)[i];
+            ((uint64_t *)data_out)[i] = (uint64_t)((uint16_t *)data_in)[i];
           return;
         case GD_FLOAT32:
           for (i = 0; i < n; i++)
-            ((float*)data_out)[i] = (float)((uint16_t*)data_in)[i];
+            ((float *)data_out)[i] = (float)((uint16_t *)data_in)[i];
           return;
         case GD_FLOAT64:
           for (i = 0; i < n; i++)
-            ((double*)data_out)[i] = (double)((uint16_t*)data_in)[i];
+            ((double *)data_out)[i] = (double)((uint16_t *)data_in)[i];
           return;
         case GD_COMPLEX64:
-          for (i = 0; i < n; i++)
-            ((float complex*)data_out)[i] =
-              (float complex)((uint16_t*)data_in)[i];
+          TO_COMPLEX(float,uint16_t);
           return;
         case GD_COMPLEX128:
-          for (i = 0; i < n; i++)
-            ((double complex*)data_out)[i] =
-              (double complex)((uint16_t*)data_in)[i];
+          TO_COMPLEX(double,uint16_t);
           return;
         default:
           _GD_SetError(D, GD_E_BAD_TYPE, out_type, NULL, 0, NULL);
@@ -273,52 +283,48 @@ void _GD_ConvertType(DIRFILE* D, const void *data_in, gd_type_t in_type,
       switch (out_type) {
         case GD_INT8:
           for (i = 0; i < n; i++)
-            ((int8_t*) data_out)[i] = (int8_t)((int32_t*)data_in)[i];
+            ((int8_t *) data_out)[i] = (int8_t)((int32_t *)data_in)[i];
           return;
         case GD_UINT8:
           for (i = 0; i < n; i++)
-            ((uint8_t*) data_out)[i] = (uint8_t)((int32_t*)data_in)[i];
+            ((uint8_t *) data_out)[i] = (uint8_t)((int32_t *)data_in)[i];
           return;
         case GD_INT16:
           for (i = 0; i < n; i++)
-            ((int16_t*)data_out)[i] = (int16_t)((int32_t*)data_in)[i];
+            ((int16_t *)data_out)[i] = (int16_t)((int32_t *)data_in)[i];
           return;
         case GD_UINT16:
           for (i = 0; i < n; i++)
-            ((uint16_t*)data_out)[i] = (uint16_t)((int32_t*)data_in)[i];
+            ((uint16_t *)data_out)[i] = (uint16_t)((int32_t *)data_in)[i];
           return;
         case GD_INT32:
           memcpy(data_out, data_in, n * sizeof(int32_t));
           return;
         case GD_UINT32:
           for (i = 0; i < n; i++)
-            ((uint32_t*)data_out)[i] = (uint32_t)((int32_t*)data_in)[i];
+            ((uint32_t *)data_out)[i] = (uint32_t)((int32_t *)data_in)[i];
           return;
         case GD_INT64:
           for (i = 0; i < n; i++)
-            ((int64_t*)data_out)[i] = (int64_t)((int32_t*)data_in)[i];
+            ((int64_t *)data_out)[i] = (int64_t)((int32_t *)data_in)[i];
           return;
         case GD_UINT64:
           for (i = 0; i < n; i++)
-            ((uint64_t*)data_out)[i] = (uint64_t)((int32_t*)data_in)[i];
+            ((uint64_t *)data_out)[i] = (uint64_t)((int32_t *)data_in)[i];
           return;
         case GD_FLOAT32:
           for (i = 0; i < n; i++)
-            ((float*)data_out)[i] = (float)((int32_t*)data_in)[i];
+            ((float *)data_out)[i] = (float)((int32_t *)data_in)[i];
           return;
         case GD_FLOAT64:
           for (i = 0; i < n; i++)
-            ((double*)data_out)[i] = (double)((int32_t*)data_in)[i];
+            ((double *)data_out)[i] = (double)((int32_t *)data_in)[i];
           return;
         case GD_COMPLEX64:
-          for (i = 0; i < n; i++)
-            ((float complex*)data_out)[i] =
-              (float complex)((int32_t*)data_in)[i];
+          TO_COMPLEX(float,int32_t);
           return;
         case GD_COMPLEX128:
-          for (i = 0; i < n; i++)
-            ((double complex*)data_out)[i] =
-              (double complex)((int32_t*)data_in)[i];
+          TO_COMPLEX(double,int32_t);
           return;
         default:
           _GD_SetError(D, GD_E_BAD_TYPE, out_type, NULL, 0, NULL);
@@ -329,52 +335,48 @@ void _GD_ConvertType(DIRFILE* D, const void *data_in, gd_type_t in_type,
       switch (out_type) {
         case GD_INT8:
           for (i = 0; i < n; i++)
-            ((int8_t*) data_out)[i] = (int8_t)((uint32_t*)data_in)[i];
+            ((int8_t *) data_out)[i] = (int8_t)((uint32_t *)data_in)[i];
           return;
         case GD_UINT8:
           for (i = 0; i < n; i++)
-            ((uint8_t*) data_out)[i] = (uint8_t)((uint32_t*)data_in)[i];
+            ((uint8_t *) data_out)[i] = (uint8_t)((uint32_t *)data_in)[i];
           return;
         case GD_INT16:
           for (i = 0; i < n; i++)
-            ((int16_t*)data_out)[i] = (int16_t)((uint32_t*)data_in)[i];
+            ((int16_t *)data_out)[i] = (int16_t)((uint32_t *)data_in)[i];
           return;
         case GD_UINT16:
           for (i = 0; i < n; i++)
-            ((uint16_t*)data_out)[i] = (uint16_t)((uint32_t*)data_in)[i];
+            ((uint16_t *)data_out)[i] = (uint16_t)((uint32_t *)data_in)[i];
           return;
         case GD_INT32:
           for (i = 0; i < n; i++)
-            ((int32_t*)data_out)[i] = (int32_t)((uint32_t*)data_in)[i];
+            ((int32_t *)data_out)[i] = (int32_t)((uint32_t *)data_in)[i];
           return;
         case GD_UINT32:
           memcpy(data_out, data_in, n * sizeof(uint32_t));
           return;
         case GD_INT64:
           for (i = 0; i < n; i++)
-            ((int64_t*)data_out)[i] = (int64_t)((uint32_t*)data_in)[i];
+            ((int64_t *)data_out)[i] = (int64_t)((uint32_t *)data_in)[i];
           return;
         case GD_UINT64:
           for (i = 0; i < n; i++)
-            ((uint64_t*)data_out)[i] = (uint64_t)((uint32_t*)data_in)[i];
+            ((uint64_t *)data_out)[i] = (uint64_t)((uint32_t *)data_in)[i];
           return;
         case GD_FLOAT32:
           for (i = 0; i < n; i++)
-            ((float*)data_out)[i] = (float)((uint32_t*)data_in)[i];
+            ((float *)data_out)[i] = (float)((uint32_t *)data_in)[i];
           return;
         case GD_FLOAT64:
           for (i = 0; i < n; i++)
-            ((double*)data_out)[i] = (double)((uint32_t*)data_in)[i];
+            ((double *)data_out)[i] = (double)((uint32_t *)data_in)[i];
           return;
         case GD_COMPLEX64:
-          for (i = 0; i < n; i++)
-            ((float complex*)data_out)[i] =
-              (float complex)((uint32_t*)data_in)[i];
+          TO_COMPLEX(float,uint32_t);
           return;
         case GD_COMPLEX128:
-          for (i = 0; i < n; i++)
-            ((double complex*)data_out)[i] =
-              (double complex)((uint32_t*)data_in)[i];
+          TO_COMPLEX(double,uint32_t);
           return;
         default:
           break;
@@ -384,52 +386,48 @@ void _GD_ConvertType(DIRFILE* D, const void *data_in, gd_type_t in_type,
       switch (out_type) {
         case GD_INT8:
           for (i = 0; i < n; i++)
-            ((int8_t*) data_out)[i] = (int64_t)((int64_t*)data_in)[i];
+            ((int8_t *) data_out)[i] = (int64_t)((int64_t *)data_in)[i];
           return;
         case GD_UINT8:
           for (i = 0; i < n; i++)
-            ((uint8_t*) data_out)[i] = (uint8_t)((int64_t*)data_in)[i];
+            ((uint8_t *) data_out)[i] = (uint8_t)((int64_t *)data_in)[i];
           return;
         case GD_INT16:
           for (i = 0; i < n; i++)
-            ((int16_t*)data_out)[i] = (int16_t)((int64_t*)data_in)[i];
+            ((int16_t *)data_out)[i] = (int16_t)((int64_t *)data_in)[i];
           return;
         case GD_UINT16:
           for (i = 0; i < n; i++)
-            ((uint16_t*)data_out)[i] = (uint16_t)((int64_t*)data_in)[i];
+            ((uint16_t *)data_out)[i] = (uint16_t)((int64_t *)data_in)[i];
           return;
         case GD_INT32:
           for (i = 0; i < n; i++)
-            ((int32_t*)data_out)[i] = (int32_t)((int64_t*)data_in)[i];
+            ((int32_t *)data_out)[i] = (int32_t)((int64_t *)data_in)[i];
           return;
         case GD_UINT32:
           for (i = 0; i < n; i++)
-            ((uint32_t*)data_out)[i] = (uint32_t)((int64_t*)data_in)[i];
+            ((uint32_t *)data_out)[i] = (uint32_t)((int64_t *)data_in)[i];
           return;
         case GD_INT64:
           memcpy(data_out, data_in, n * sizeof(int64_t));
           return;
         case GD_UINT64:
           for (i = 0; i < n; i++)
-            ((uint64_t*)data_out)[i] = (uint64_t)((int64_t*)data_in)[i];
+            ((uint64_t *)data_out)[i] = (uint64_t)((int64_t *)data_in)[i];
           return;
         case GD_FLOAT32:
           for (i = 0; i < n; i++)
-            ((float*)data_out)[i] = (float)((int64_t*)data_in)[i];
+            ((float *)data_out)[i] = (float)((int64_t *)data_in)[i];
           return;
         case GD_FLOAT64:
           for (i = 0; i < n; i++)
-            ((double*)data_out)[i] = (double)((int64_t*)data_in)[i];
+            ((double *)data_out)[i] = (double)((int64_t *)data_in)[i];
           return;
         case GD_COMPLEX64:
-          for (i = 0; i < n; i++)
-            ((float complex*)data_out)[i] =
-              (float complex)((int64_t*)data_in)[i];
+          TO_COMPLEX(float,int64_t);
           return;
         case GD_COMPLEX128:
-          for (i = 0; i < n; i++)
-            ((double complex*)data_out)[i] =
-              (double complex)((int64_t*)data_in)[i];
+          TO_COMPLEX(double,int64_t);
           return;
         default:
           _GD_SetError(D, GD_E_BAD_TYPE, out_type, NULL, 0, NULL);
@@ -440,52 +438,48 @@ void _GD_ConvertType(DIRFILE* D, const void *data_in, gd_type_t in_type,
       switch (out_type) {
         case GD_INT8:
           for (i = 0; i < n; i++)
-            ((int8_t*) data_out)[i] = (int8_t)((uint64_t*)data_in)[i];
+            ((int8_t *) data_out)[i] = (int8_t)((uint64_t *)data_in)[i];
           return;
         case GD_UINT8:
           for (i = 0; i < n; i++)
-            ((uint8_t*) data_out)[i] = (uint8_t)((uint64_t*)data_in)[i];
+            ((uint8_t *) data_out)[i] = (uint8_t)((uint64_t *)data_in)[i];
           return;
         case GD_INT16:
           for (i = 0; i < n; i++)
-            ((int16_t*)data_out)[i] = (int16_t)((uint64_t*)data_in)[i];
+            ((int16_t *)data_out)[i] = (int16_t)((uint64_t *)data_in)[i];
           return;
         case GD_UINT16:
           for (i = 0; i < n; i++)
-            ((uint16_t*)data_out)[i] = (uint16_t)((uint64_t*)data_in)[i];
+            ((uint16_t *)data_out)[i] = (uint16_t)((uint64_t *)data_in)[i];
           return;
         case GD_INT32:
           for (i = 0; i < n; i++)
-            ((int32_t*)data_out)[i] = (int32_t)((uint64_t*)data_in)[i];
+            ((int32_t *)data_out)[i] = (int32_t)((uint64_t *)data_in)[i];
           return;
         case GD_UINT32:
           for (i = 0; i < n; i++)
-            ((uint32_t*)data_out)[i] = (uint32_t)((uint64_t*)data_in)[i];
+            ((uint32_t *)data_out)[i] = (uint32_t)((uint64_t *)data_in)[i];
           return;
         case GD_INT64:
           for (i = 0; i < n; i++)
-            ((int64_t*)data_out)[i] = (int64_t)((uint64_t*)data_in)[i];
+            ((int64_t *)data_out)[i] = (int64_t)((uint64_t *)data_in)[i];
           return;
         case GD_UINT64:
           memcpy(data_out, data_in, n * sizeof(uint64_t));
           return;
         case GD_FLOAT32:
           for (i = 0; i < n; i++)
-            ((float*)data_out)[i] = (float)((uint64_t*)data_in)[i];
+            ((float *)data_out)[i] = (float)((uint64_t *)data_in)[i];
           return;
         case GD_FLOAT64:
           for (i = 0; i < n; i++)
-            ((double*)data_out)[i] = (double)((uint64_t*)data_in)[i];
+            ((double *)data_out)[i] = (double)((uint64_t *)data_in)[i];
           return;
         case GD_COMPLEX64:
-          for (i = 0; i < n; i++)
-            ((float complex*)data_out)[i] =
-              (float complex)((uint64_t*)data_in)[i];
+          TO_COMPLEX(float,uint64_t);
           return;
         case GD_COMPLEX128:
-          for (i = 0; i < n; i++)
-            ((double complex*)data_out)[i] =
-              (double complex)((uint64_t*)data_in)[i];
+          TO_COMPLEX(double,uint64_t);
           return;
         default:
           _GD_SetError(D, GD_E_BAD_TYPE, out_type, NULL, 0, NULL);
@@ -496,51 +490,48 @@ void _GD_ConvertType(DIRFILE* D, const void *data_in, gd_type_t in_type,
       switch (out_type) {
         case GD_INT8:
           for (i = 0; i < n; i++)
-            ((int8_t*) data_out)[i] = (int8_t)((float*)data_in)[i];
+            ((int8_t *) data_out)[i] = (int8_t)((float *)data_in)[i];
           return;
         case GD_UINT8:
           for (i = 0; i < n; i++)
-            ((uint8_t*) data_out)[i] = (uint8_t)((float*)data_in)[i];
+            ((uint8_t *) data_out)[i] = (uint8_t)((float *)data_in)[i];
           return;
         case GD_INT16:
           for (i = 0; i < n; i++)
-            ((int16_t*)data_out)[i] = (int16_t)((float*)data_in)[i];
+            ((int16_t *)data_out)[i] = (int16_t)((float *)data_in)[i];
           return;
         case GD_UINT16:
           for (i = 0; i < n; i++)
-            ((uint16_t*)data_out)[i] = (uint16_t)((float*)data_in)[i];
+            ((uint16_t *)data_out)[i] = (uint16_t)((float *)data_in)[i];
           return;
         case GD_INT32:
           for (i = 0; i < n; i++)
-            ((int32_t*)data_out)[i] = (int32_t)((float*)data_in)[i];
+            ((int32_t *)data_out)[i] = (int32_t)((float *)data_in)[i];
           return;
         case GD_UINT32:
           for (i = 0; i < n; i++)
-            ((uint32_t*)data_out)[i] = (uint32_t)((float*)data_in)[i];
+            ((uint32_t *)data_out)[i] = (uint32_t)((float *)data_in)[i];
           return;
         case GD_INT64:
           for (i = 0; i < n; i++)
-            ((int64_t*)data_out)[i] = (int64_t)((float*)data_in)[i];
+            ((int64_t *)data_out)[i] = (int64_t)((float *)data_in)[i];
           return;
         case GD_UINT64:
           for (i = 0; i < n; i++)
-            ((uint64_t*)data_out)[i] = (uint64_t)((float*)data_in)[i];
+            ((uint64_t *)data_out)[i] = (uint64_t)((float *)data_in)[i];
           return;
         case GD_FLOAT32:
           memcpy(data_out, data_in, n * sizeof(float));
           return;
         case GD_FLOAT64:
           for (i = 0; i < n; i++)
-            ((double*)data_out)[i] = (double)((float*)data_in)[i];
+            ((double *)data_out)[i] = (double)((float *)data_in)[i];
           return;
         case GD_COMPLEX64:
-          for (i = 0; i < n; i++)
-            ((float complex*)data_out)[i] = (float complex)((float*)data_in)[i];
+          TO_COMPLEX(float,float);
           return;
         case GD_COMPLEX128:
-          for (i = 0; i < n; i++)
-            ((double complex*)data_out)[i] =
-              (double complex)((float*)data_in)[i];
+          TO_COMPLEX(double,float);
           return;
         default:
           _GD_SetError(D, GD_E_BAD_TYPE, out_type, NULL, 0, NULL);
@@ -551,52 +542,48 @@ void _GD_ConvertType(DIRFILE* D, const void *data_in, gd_type_t in_type,
       switch (out_type) {
         case GD_INT8:
           for (i = 0; i < n; i++)
-            ((int8_t*) data_out)[i] = (int8_t)((double*)data_in)[i];
+            ((int8_t *) data_out)[i] = (int8_t)((double *)data_in)[i];
           return;
         case GD_UINT8:
           for (i = 0; i < n; i++)
-            ((uint8_t*) data_out)[i] = (uint8_t)((double*)data_in)[i];
+            ((uint8_t *) data_out)[i] = (uint8_t)((double *)data_in)[i];
           return;
         case GD_INT16:
           for (i = 0; i < n; i++)
-            ((int16_t*)data_out)[i] = (int16_t)((double*)data_in)[i];
+            ((int16_t *)data_out)[i] = (int16_t)((double *)data_in)[i];
           return;
         case GD_UINT16:
           for (i = 0; i < n; i++)
-            ((uint16_t*)data_out)[i] = (uint16_t)((double*)data_in)[i];
+            ((uint16_t *)data_out)[i] = (uint16_t)((double *)data_in)[i];
           return;
         case GD_INT32:
           for (i = 0; i < n; i++)
-            ((int32_t*)data_out)[i] = (int32_t)((double*)data_in)[i];
+            ((int32_t *)data_out)[i] = (int32_t)((double *)data_in)[i];
           return;
         case GD_UINT32:
           for (i = 0; i < n; i++)
-            ((uint32_t*)data_out)[i] = (int32_t)((double*)data_in)[i];
+            ((uint32_t *)data_out)[i] = (int32_t)((double *)data_in)[i];
           return;
         case GD_INT64:
           for (i = 0; i < n; i++)
-            ((int64_t*)data_out)[i] = (int64_t)((double*)data_in)[i];
+            ((int64_t *)data_out)[i] = (int64_t)((double *)data_in)[i];
           return;
         case GD_UINT64:
           for (i = 0; i < n; i++)
-            ((uint64_t*)data_out)[i] = (uint64_t)((double*)data_in)[i];
+            ((uint64_t *)data_out)[i] = (uint64_t)((double *)data_in)[i];
           return;
         case GD_FLOAT32:
           for (i = 0; i < n; i++)
-            ((float*)data_out)[i] = (float)((double*)data_in)[i];
+            ((float *)data_out)[i] = (float)((double *)data_in)[i];
           return;
         case GD_FLOAT64:
           memcpy(data_out, data_in, n * sizeof(double));
           return;
         case GD_COMPLEX64:
-          for (i = 0; i < n; i++)
-            ((float complex*)data_out)[i] =
-              (float complex)((double*)data_in)[i];
+          TO_COMPLEX(float,double);
           return;
         case GD_COMPLEX128:
-          for (i = 0; i < n; i++)
-            ((double complex*)data_out)[i] =
-              (double complex)((double*)data_in)[i];
+          TO_COMPLEX(double,double);
           return;
         default:
           _GD_SetError(D, GD_E_BAD_TYPE, out_type, NULL, 0, NULL);
@@ -606,52 +593,41 @@ void _GD_ConvertType(DIRFILE* D, const void *data_in, gd_type_t in_type,
     case GD_COMPLEX64:
       switch (out_type) {
         case GD_INT8:
-          for (i = 0; i < n; i++)
-            ((int8_t*) data_out)[i] = (int8_t)((float complex*)data_in)[i];
+          FROM_COMPLEX(int8_t,float);
           return;
         case GD_UINT8:
-          for (i = 0; i < n; i++)
-            ((uint8_t*) data_out)[i] = (uint8_t)((float complex*)data_in)[i];
+          FROM_COMPLEX(uint8_t,float);
           return;
         case GD_INT16:
-          for (i = 0; i < n; i++)
-            ((int16_t*)data_out)[i] = (int16_t)((float complex*)data_in)[i];
+          FROM_COMPLEX(int16_t,float);
           return;
         case GD_UINT16:
-          for (i = 0; i < n; i++)
-            ((uint16_t*)data_out)[i] = (uint16_t)((float complex*)data_in)[i];
+          FROM_COMPLEX(uint16_t,float);
           return;
         case GD_INT32:
-          for (i = 0; i < n; i++)
-            ((int32_t*)data_out)[i] = (int32_t)((float complex*)data_in)[i];
+          FROM_COMPLEX(int32_t,float);
           return;
         case GD_UINT32:
-          for (i = 0; i < n; i++)
-            ((uint32_t*)data_out)[i] = (int32_t)((float complex*)data_in)[i];
+          FROM_COMPLEX(uint32_t,float);
           return;
         case GD_INT64:
-          for (i = 0; i < n; i++)
-            ((int64_t*)data_out)[i] = (int64_t)((float complex*)data_in)[i];
+          FROM_COMPLEX(int64_t,float);
           return;
         case GD_UINT64:
-          for (i = 0; i < n; i++)
-            ((uint64_t*)data_out)[i] = (uint64_t)((float complex*)data_in)[i];
+          FROM_COMPLEX(uint64_t,float);
           return;
         case GD_FLOAT32:
-          for (i = 0; i < n; i++)
-            ((float*)data_out)[i] = (float)((float complex*)data_in)[i];
+          FROM_COMPLEX(float,float);
           return;
         case GD_FLOAT64:
-          for (i = 0; i < n; i++)
-            ((double*)data_out)[i] = (double)((float complex*)data_in)[i];
+          FROM_COMPLEX(double,float);
           return;
         case GD_COMPLEX64:
-          memcpy(data_out, data_in, n * sizeof(float complex));
+          memcpy(data_out, data_in, n * 2 * sizeof(float));
           return;
         case GD_COMPLEX128:
-          for (i = 0; i < n; i++)
-            ((double complex*)data_out)[i] =
-              (double complex)((float complex*)data_in)[i];
+          for (i = 0; i < 2 * n; i++)
+            ((double *)data_out)[i] = (double)((float *)data_in)[i];
           return;
         default:
           _GD_SetError(D, GD_E_BAD_TYPE, out_type, NULL, 0, NULL);
@@ -660,52 +636,41 @@ void _GD_ConvertType(DIRFILE* D, const void *data_in, gd_type_t in_type,
     case GD_COMPLEX128:
       switch (out_type) {
         case GD_INT8:
-          for (i = 0; i < n; i++)
-            ((int8_t*) data_out)[i] = (int8_t)((double complex*)data_in)[i];
+          FROM_COMPLEX(int8_t,double);
           return;
         case GD_UINT8:
-          for (i = 0; i < n; i++)
-            ((uint8_t*) data_out)[i] = (uint8_t)((double complex*)data_in)[i];
+          FROM_COMPLEX(uint8_t,double);
           return;
         case GD_INT16:
-          for (i = 0; i < n; i++)
-            ((int16_t*)data_out)[i] = (int16_t)((double complex*)data_in)[i];
+          FROM_COMPLEX(int16_t,double);
           return;
         case GD_UINT16:
-          for (i = 0; i < n; i++)
-            ((uint16_t*)data_out)[i] = (uint16_t)((double complex*)data_in)[i];
+          FROM_COMPLEX(uint16_t,double);
           return;
         case GD_INT32:
-          for (i = 0; i < n; i++)
-            ((int32_t*)data_out)[i] = (int32_t)((double complex*)data_in)[i];
+          FROM_COMPLEX(int32_t,double);
           return;
         case GD_UINT32:
-          for (i = 0; i < n; i++)
-            ((uint32_t*)data_out)[i] = (int32_t)((double complex*)data_in)[i];
+          FROM_COMPLEX(uint32_t,double);
           return;
         case GD_INT64:
-          for (i = 0; i < n; i++)
-            ((int64_t*)data_out)[i] = (int64_t)((double complex*)data_in)[i];
+          FROM_COMPLEX(int64_t,double);
           return;
         case GD_UINT64:
-          for (i = 0; i < n; i++)
-            ((uint64_t*)data_out)[i] = (uint64_t)((double complex*)data_in)[i];
+          FROM_COMPLEX(uint64_t,double);
           return;
         case GD_FLOAT32:
-          for (i = 0; i < n; i++)
-            ((float*)data_out)[i] = (float)((double complex*)data_in)[i];
+          FROM_COMPLEX(float,double);
           return;
         case GD_FLOAT64:
-          for (i = 0; i < n; i++)
-            ((double*)data_out)[i] = (double)((double complex*)data_in)[i];
+          FROM_COMPLEX(double,double);
           return;
         case GD_COMPLEX64:
-          for (i = 0; i < n; i++)
-            ((float complex*)data_out)[i] =
-              (float complex)((double complex*)data_in)[i];
+          for (i = 0; i < 2 * n; i++)
+            ((float *)data_out)[i] = (float)((double *)data_in)[i];
           return;
         case GD_COMPLEX128:
-          memcpy(data_out, data_in, n * sizeof(double complex));
+          memcpy(data_out, data_in, 2 * n * sizeof(double));
           return;
         default:
           _GD_SetError(D, GD_E_BAD_TYPE, out_type, NULL, 0, NULL);
@@ -719,4 +684,4 @@ void _GD_ConvertType(DIRFILE* D, const void *data_in, gd_type_t in_type,
 }
 
 /* vim: ts=2 sw=2 et tw=80
- */
+*/
